'use strict';

/* For ESLint: List any global identifiers used in this file below */
/* global browser, translate, BG, browser, optionalSettings:true,
getStorageCookie, setStorageCookie, THIRTY_MINUTES_IN_MILLISECONDS,
ThemeManager, updateAcceptableAdsUI */

// Contains all filter lists and their respective containers.
let filterListSections = {};

// Utility class for filter lists.
function FilterListUtil() {}

// Utility class for custom filter list upload box.
function CustomFilterListUploadUtil() {}

// Utility class for Subscriptions.
function SubscriptionUtil() {}

// Utility class for the language select.
function LanguageSelectUtil() {}

// Helper function to translate filter list ids correctly
function translateIDs(id) {
  const idsWith2 = ['warning_removal', 'annoyances', 'antisocial'];
  if (idsWith2.includes(id)) {
    return translate(`filter${id}2`);
  }
  return translate(`filter${id}`);
}

browser.runtime.onMessage.addListener((request, sender, sendResponse) => {
  if (request && request.message !== 'addSubscription') {
    return;
  }

  if (!request) {
    return;
  }

  const title = request.title || request.url;
  const { url } = request;
  const translatedMsg = translate('subscribeconfirm', title);
  // eslint-disable-next-line no-alert
  if (window.confirm(translatedMsg)) {
    const existingFilterList = FilterListUtil.checkUrlForExistingFilterList(url);
    if (existingFilterList) {
      FilterListUtil.cachedSubscriptions[existingFilterList.id].subscribed = true;
      CustomFilterListUploadUtil.updateExistingFilterList(existingFilterList);
    } else if (/^https?:\/\/[^<]+$/.test(url)) {
      CustomFilterListUploadUtil.performUpload(url, `url:${url}`, title);
    } else {
      // eslint-disable-next-line no-alert
      alert(translate('failedtofetchfilter'));
    }
  }

  sendResponse({});
});

function getDefaultFilterUI(filterList, checkboxID, filterListType) {
  const isAcceptableAds = BG.isAcceptableAds(filterList);
  const isLanguageFilter = filterListType === 'languageFilterList';
  const aaPrivacy = FilterListUtil.cachedSubscriptions.acceptable_ads_privacy;
  let isSelected = filterList.subscribed;
  let filterUrl = filterList.url;

  if (isAcceptableAds && aaPrivacy.subscribed) {
    isSelected = true;
    filterUrl = aaPrivacy.url;
  }

  let displayStyle = 'block';
  if (isLanguageFilter && !isSelected) {
    displayStyle = 'none';
  }

  const $checkBox = $('<input>')
    .attr('type', 'checkbox')
    .attr('id', checkboxID)
    .css('margin-left', '2px')
    .prop('checked', isSelected)
    .addClass('filter_list_control');

  let labelText = filterList.label || filterList.title;
  const labelMax = 42;
  if (!labelText) {
    if (filterList.url.length > labelMax) {
      labelText = `${filterList.url.substr(0, labelMax)}...`;
    } else {
      labelText = filterList.url;
    }
  }
  const $label = $('<label>')
    .text(labelText)
    .attr('title', filterList.url)
    .attr('for', checkboxID);

  const $linkIcon = $('<i>')
    .addClass('material-icons')
    .addClass('md-24')
    .text('list');
  const $link = $('<a></a>')
    .css('display', $('#btnShowLinks').prop('disabled') ? 'inline' : 'none')
    .attr('target', '_blank')
    .attr('class', 'linkToList')
    .attr('alt', filterList.label || filterList.title)
    .attr('href', filterUrl)
    .append($linkIcon);

  const $infospan = $('<span>')
    .addClass('subscription_info')
    .addClass('notes')
    .text(filterList.subscribed && !filterList.lastUpdate ? (translate('fetchinglabel')) : '');

  const $whatThisAria = $('<span>whats_this</span>')
    .addClass('sr-only')
    .text(translate('whats_this'));

  const $whatThisIcon = $('<i>')
    .addClass('material-icons')
    .addClass('md-18')
    .text('live_help');

  const $whatsthislink = $('<a>')
    .addClass('notes')
    .attr('href', 'https://help.getadblock.com/solution/articles/6000142821-what-is-easylist-lite-')
    .attr('target', '_blank')
    .append($whatThisIcon)
    .append($whatThisAria);

  const $removeFilterListLabel = filterList.userSubmitted ? $('<a>')
    .css('font-size', '10px')
    .css('display', filterList.subscribed ? 'none' : 'inline')
    .css('padding-left', '10px')
    .attr('href', '#')
    .addClass('remove_filter_list')
    .text(translate('removefromlist'))
    .click(function removeFilterListLabelClickFunction(event) {
      event.preventDefault();
      const $parent = $(this).parent();
      const id = $parent.attr('name');
      SubscriptionUtil.unsubscribe(id, true);
      $parent.remove();
    }) : null;

  const $extraInformation = $('<div>')
    .addClass('extra-info')
    .text(translate(`filter_${filterList.id}_explained`));

  const $checkBoxWrap = $('<div>')
    .addClass('check-box')
    .append($checkBox)
    .append($label);

  const $fitlerListItem = $('<li>')
    .addClass('subscription')
    .addClass(filterListType)
    .attr('name', filterList.id)
    .css('display', displayStyle)
    .append($checkBoxWrap)
    .append(filterList.id === 'easylist_lite' ? $whatsthislink : null)
    .append($link)
    .append($infospan)
    .append($removeFilterListLabel)
    .append($extraInformation);

  return {
    checkbox: $checkBox,
    filterList: $fitlerListItem,
  };
}

function getToggleFilterUI(filterList, checkboxID) {
  const $checkBox = $('<input>')
    .addClass('filter_list_control')
    .attr('type', 'checkbox')
    .attr('id', checkboxID)
    .prop('checked', !!filterList.subscribed);

  const $toggleItem = $('<li>')
    .addClass('subscription')
    .attr('name', filterList.id)
    .append($('<label>')
      .addClass('switch')
      .attr('title', filterList.url)
      .attr('for', checkboxID)
      .append($checkBox)
      .append($('<span>')
        .addClass('slider')
        .addClass('round')))
    .append($('<span>')
      .text(translate('acceptable_ads_privacy')));
  return {
    checkbox: $checkBox,
    filterList: $toggleItem,
  };
}

// Represents the checkbox controlling subscription to a specific filter list,
// as well as the UI that goes along with it (e.g. status label and removal link.)
// Inputs:
//   filterList:object - An entry from a filterListSections array.
//   filterListType:string - A key from filterListSections describing the section
//       where this checkbox will appear, e.g. 'languageFilterList'.
//   index:int - The position in the section where this checkbox should appear.
//   container:jQuery - The DOM element for the section in which this checkbox
//       should appear.
function CheckboxForFilterList(filterList, filterListType, index, container) {
  this.container = container;
  this.filterList = filterList;
  this.id = `${filterListType}_${index}`;

  let uiItems;
  if (BG.isAcceptableAdsPrivacy(filterList)) {
    uiItems = getToggleFilterUI(this.filterList, this.id);
  } else {
    uiItems = getDefaultFilterUI(this.filterList, this.id, filterListType);
  }
  this.checkBox = uiItems.checkbox;
  this.filterListItem = uiItems.filterList;
}

// Represents the option for language filter lists inside the language select.
// Inputs:
//   filterList:object - An entry from a filterListSections array.
//   index:int - The position in the language select where this option should appear.
function OptionForFilterList(filterList, index) {
  this.filterList = filterList;
  this.index = index;

  this.option = $('<option>', {
    value: this.filterList.id,
    text: this.filterList.label,
  }).data('index', this.index);
}

OptionForFilterList.prototype = {
  // Returns the option attribute.
  get() {
    return this.option;
  },
};

// add a checkbox to for the user to indicate if they wish to be notified of blocked malware
function addMalwareNotificationDiv() {
  if (document.getElementById('malware-notification-message-div')) {
    return;
  } // already exists, don't add it again.
  browser.storage.local.get('malware-notification', (response) => {
    const notify = response['malware-notification'];
    const newDiv = $('<div>')
      .attr('id', 'malware-notification-message-div');
    const newInput = $('<input />')
      .attr('type', 'checkbox')
      .attr('id', 'malware-notification-message')
      .css('margin-left', '25px')
      .prop('checked', notify ? true : null);
    const newLabel = $('<label>')
      .text(translate('malwarenotificationcheckboxmessage'))
      .attr('for', 'malware-notification-message');
    const betaLabel = $('<label>')
      .text(translate('betalabel'))
      .attr('i18n', 'betalabel')
      .css('padding-left', '5px');
    newDiv.append(newInput).append(newLabel).append(betaLabel);

    $("div[name='malware']").after(newDiv);

    $('#malware-notification-message').click(function malwareNotificationClickFunction() {
      const checked = $(this).is(':checked');
      browser.storage.local.set({ 'malware-notification': checked });
    });
  });
}

CheckboxForFilterList.prototype = {
  // Bind checkbox on change event to handle subscribing and unsubscribing to
  // different filter lists.
  bindActions() {
    const that = this;
    this.checkBox
      .change(function checkBoxChangeFunction() {
        const parent = $(this).parent().parent();
        const checked = $(this).is(':checked');
        const id = parent.attr('name');
        if (checked) {
          if (!SubscriptionUtil.validateOverSubscription(that.filterList)) {
            $(this).prop('checked', false);
            return;
          }
          if (BG.isAcceptableAdsPrivacy(that.filterList)) {
            $('.subscription_info', '[name=acceptable_ads]').text(translate('fetchinglabel'));
          } else {
            $('.subscription_info', parent).text(translate('fetchinglabel'));
          }
          SubscriptionUtil.subscribe(id);
          delete FilterListUtil.cachedSubscriptions[id].unsubscribed;
        } else {
          if (!SubscriptionUtil.validateUnderSubscription(that.filterList)) {
            $(this).prop('checked', true);
            return;
          }
          SubscriptionUtil.unsubscribe(id, false);
          $('.subscription_info', parent)
            .text(translate('unsubscribedlabel'));
          delete FilterListUtil.cachedSubscriptions[id].subscribed;
        }
        $('.remove_filter_list', parent)
          .css('display', checked ? 'none' : 'inline');
        // if the checkbox clicked is the malware
        // add a checkbox to for the user to indicate if they wish to be notified of blocked malware
        if (id && id === 'malware' && checked) {
          addMalwareNotificationDiv();
        } else if (id && id === 'malware' && !checked) {
          $('#malware-notification-message-div').remove();
          browser.storage.local.set({ 'malware-notification': false });
        }

        if (parent.attr('class').indexOf('languageFilterList') > -1) {
          parent.toggle(500);
          if (!checked) {
            const $this = $(this);
            const index = $this.attr('id').split('_')[1];
            const entry = filterListSections.languageFilterList.array[index];
            if (!Object.prototype.hasOwnProperty.call(entry, 'label')) {
              entry.label = translateIDs(id);
            }

            const option = new OptionForFilterList(entry, index);
            LanguageSelectUtil.insertOption(option.get(), index);
          }
        }
      });
  },

  // Append filter list checkbox to the container.
  // Inputs:
  //   isChecked:boolean - Flag that will indicate that this checkbox is checked by default.
  createCheckbox(isChecked) {
    this.container.append(this.filterListItem);

    this.bindActions();

    if (isChecked) {
      this.checkBox.prop('checked', 'checked');
      this.checkBox.trigger('change');
    }
  },
};

// This class is in charge of creating the display per filter list type.
// Inputs:
//   filterListSection:object - One of the objects in filterListSections.
//   filterListType:string - Will serve as the identifier for the corresponding filter list,
//      use the keys in filterListSections as its value.
function SectionHandler(filterListSection, filterListType) {
  this.cachedSubscriptions = filterListSection.array;
  this.section = filterListSection.container;
  this.filterListType = filterListType;
}

SectionHandler.prototype = {
  // Organize each container for checkboxes.
  organize() {
    for (let i = 0; i < this.cachedSubscriptions.length; i += 1) {
      const filterList = this.cachedSubscriptions[i];
      const checkbox = new CheckboxForFilterList(filterList, this.filterListType, i, this.section);
      checkbox.createCheckbox();
    }
  },
  // Initialization call. Calls organize to start displaying things.
  initSection() {
    this.organize();
  },
};

FilterListUtil.sortFilterListArrays = function sortFilterListArraysFunction() {
  // Sort alphabetically
  for (const filterList in filterListSections) {
    filterListSections[filterList].array.sort((a, b) => {
      let returnVal;
      if (a.label > b.label) {
        returnVal = 1;
      } else if (a.label === b.label) {
        returnVal = 0;
      } else {
        returnVal = -1;
      }
      return returnVal;
    });
  }
  // Move AA privacy after AA
  const section = filterListSections.adblockFilterList.array;
  const aaPrivacyIndex = section.findIndex(filterList => BG.isAcceptableAdsPrivacy(filterList));
  if (aaPrivacyIndex > -1) {
    const aaPrivacyFilter = section.splice(aaPrivacyIndex, 1)[0];
    const aaIndex = section.findIndex(filterList => BG.isAcceptableAds(filterList));
    if (aaIndex > -1) {
      section.splice(aaIndex + 1, 0, aaPrivacyFilter);
    }
  }
};
// Prepare filterListSections.
// Inputs:
//    subs:object - Map for subscription lists taken from the background.
FilterListUtil.getFilterListType = function getFilterListTypeFunction(filterList) {
  let filterListType;
  if (filterList.id === 'adblock_custom'
        || filterList.id === 'acceptable_ads'
        || filterList.id === 'easylist_lite'
        || filterList.id === 'anticircumvent'
        || filterList.id === 'easylist'
        || filterList.id === 'acceptable_ads_privacy') {
    filterListType = 'adblockFilterList';
  } else if (filterList.id === 'easyprivacy'
        || filterList.id === 'antisocial'
        || filterList.id === 'malware'
        || filterList.id === 'bitcoin_mining_protection'
        || filterList.id === 'annoyances'
        || filterList.id === 'warning_removal') {
    filterListType = 'otherFilterList';
  } else if (filterList.userSubmitted) {
    filterListType = 'customFilterList';
  } else {
    filterListType = 'languageFilterList';
  }

  return filterListType;
};

FilterListUtil.prepareSubscriptions = function prepareSubscriptionsFunction(subs) {
  FilterListUtil.cachedSubscriptions = subs;
  for (const id in subs) {
    const entry = subs[id];
    // An exception is thrown if the translate msg isn't found.
    // Since, the user can enter custom filter URLs, translate will fail to find it.
    try {
      entry.label = translateIDs(id);
    } catch (ex) {
      // Ignore error
      entry.label = '';
    }

    entry.id = id;
    const filterListType = FilterListUtil.getFilterListType(entry);
    filterListSections[filterListType].array.push(entry);
  }

  FilterListUtil.sortFilterListArrays();
};

// Returns the subscription info object for the custom filter list specified by |url|,
// or undefined if that custom filter list does not exist.
// Inputs:
//   url:string - Url for uploaded custom filter list.
FilterListUtil.checkUrlForExistingFilterList = function checkUrlForExistingFilterListFunction(url) {
  const { cachedSubscriptions } = FilterListUtil;
  for (const id in cachedSubscriptions) {
    if (url === cachedSubscriptions[id].url) {
      return cachedSubscriptions[id];
    }
  }
  return undefined;
};

// Updates info text for each filter list.
FilterListUtil.updateSubscriptionInfoAll = function updateSubscriptionInfoAllFunction() {
  const { cachedSubscriptions } = FilterListUtil;
  for (const id in cachedSubscriptions) {
    const subscription = cachedSubscriptions[id];
    let div = $(`[name='${id}']`);
    div = BG.isAcceptableAdsPrivacy(subscription) ? $('[name=acceptable_ads]') : div;

    const infoLabel = $('.subscription_info', div);
    infoLabel.addClass('notes');
    let text = infoLabel.text();

    const { lastUpdate } = subscription;

    // If filter list is invalid or unsubscribed, skip it.
    if (infoLabel.text() !== translate('invalidListUrl')
          && infoLabel.text() !== translate('unsubscribedlabel')) {
      if (subscription.lastUpdateFailedAt) {
        if (subscription.userSubmitted
              && translate('failedtofetchfilter') === infoLabel.text()) {
          text = translate('invalidListUrl');
          $('input', div).attr('disabled', 'disabled');
        } else {
          text = translate('failedtofetchfilter');
        }
      } else if (lastUpdate) {
        const howLongAgo = Date.now() - lastUpdate;
        const seconds = Math.round(howLongAgo / 1000);
        const minutes = Math.round(seconds / 60);
        const hours = Math.round(minutes / 60);
        const days = Math.round(hours / 24);
        text = '';
        if (seconds < 10) {
          text += translate('updatedrightnow');
        } else if (seconds < 60) {
          text += translate('updatedsecondsago', [seconds]);
        } else if (minutes === 1) {
          text += translate('updatedminuteago');
        } else if (minutes < 60) {
          text += translate('updatedminutesago', [minutes]);
        } else if (hours === 1) {
          text += translate('updatedhourago');
        } else if (hours < 24) {
          text += translate('updatedhoursago', [hours]);
        } else if (days === 1) {
          text += translate('updateddayago');
        } else {
          text += translate('updateddaysago', [days]);
        }
      }

      infoLabel.text(text);
    }
  }
};

// Update checkbox for the filter list according to it's current state.
// Inputs:
//    filterList:object - Filter list that owns the check box to be updated.
//    id:String - Id of the filter list to be updated, also the name of the
//                containing div in display.
FilterListUtil.updateCheckbox = function updateCheckboxFunction(filterList, id) {
  const subscription = FilterListUtil.cachedSubscriptions[id];
  let $containingDiv = $(`li[name="${id}"]`);
  const $checkbox = $containingDiv.find('input');
  const $filterLink = $containingDiv.find('a.linkToList');
  const aaPrivacy = FilterListUtil.cachedSubscriptions.acceptable_ads_privacy;
  const subInfo = filterList.subscribed ? translate('fetchinglabel') : translate('unsubscribedlabel');

  if (BG.isAcceptableAdsPrivacy(subscription) && !aaPrivacy.subscribed) {
    return;
  }

  if (BG.isAcceptableAdsPrivacy(subscription)) {
    $containingDiv = $('li[name=acceptable_ads]');
  }

  if (BG.isAcceptableAds(subscription) && aaPrivacy.subscribed) {
    $checkbox.prop('checked', true);
    $filterLink.attr('href', aaPrivacy.url);
    return;
  }

  if (BG.isAcceptableAds(subscription)) {
    $filterLink.attr('href', filterList.url);
  }

  $checkbox.prop('checked', filterList.subscribed);
  $('.subscription_info', $containingDiv).text(subInfo);
};

// Insert option at specified index in the language select.
// Inputs:
//   option:OptionForFilterList - Option to be inserted.
//   index:int - Where to insert the option.
LanguageSelectUtil.insertOption = function insertOptionFunction(option, index) {
  const $languageSelect = $('#language_select');
  const options = $languageSelect.find('option');
  let i;
  for (i = 0; i < options.length; i += 1) {
    const listOptionIndex = options.eq(i).data('index');
    if (listOptionIndex && parseInt(listOptionIndex, 10) > parseInt(index, 10)) {
      break;
    }
  }

  if (options.eq(i).length > 0) {
    options.eq(i).before(option);
  } else {
    $languageSelect.append(option);
  }
};

// Puts all unsubscribed language filter lists into the language select,
// and binds an onChange event on the select to subscribe to the selected
// filter list.
LanguageSelectUtil.init = function initFunction() {
  const languageFilterListArr = filterListSections.languageFilterList.array;
  for (let i = 0; i < languageFilterListArr.length; i += 1) {
    const languageFilterList = languageFilterListArr[i];
    if (!languageFilterList.subscribed) {
      const option = new OptionForFilterList(languageFilterList, i);
      LanguageSelectUtil.insertOption(option.get(), i);
    }
  }

  $('#language_select').change(function languageSelectChangeFunction() {
    const $this = $(this);
    const selectedOption = $this.find(':selected');
    const index = $(selectedOption).data('index');
    const entry = languageFilterListArr[index];
    if (entry) {
      $this.find('option:first').prop('selected', 'selected');
      selectedOption.remove();
      const $checkbox = $(`[name='${entry.id}']`).find('input');
      $checkbox.prop('checked', true);
      $checkbox.trigger('change');
    }
  });
};

// Trigger change event to language select using one of the entries.
// Input:
//   filterList:object - Filter list to select.
LanguageSelectUtil.triggerChange = function triggerChangeFunction(filterList) {
  const $languageSelect = $('#language_select');
  $languageSelect.val(filterList.id);
  $languageSelect.trigger('change');
};

// Returns true if the user knows what they are doing, subscribing to many
// filter lists.
SubscriptionUtil.validateOverSubscription = function valOverSubscription(clickedFitlerList) {
  const totalSelected = $('.filter_list_control:checked').length;
  const aaSelected = $('[name=acceptable_ads] :checked').length;
  const aaPrivacyClicked = BG.isAcceptableAdsPrivacy(clickedFitlerList);

  if (totalSelected <= 6) {
    return true;
  }

  // No need to show the dialog window because we change from AA to
  // AA Privacy without altering the total number of subscriptions
  if (aaSelected && aaPrivacyClicked) {
    return true;
  }

  if (optionalSettings.show_advanced_options) {
    // In case of an advanced user, only warn once every 30 minutes, even
    // if the options page wasn't open all the time. 30 minutes = 1/48 day
    if (getStorageCookie('noOversubscriptionWarning')) {
      return true;
    }
    setStorageCookie('noOversubscriptionWarning', 'true', THIRTY_MINUTES_IN_MILLISECONDS);
  }

  // eslint-disable-next-line no-alert
  return window.confirm(translate('you_know_thats_a_bad_idea_right'));
};

// Returns true if the user knows what they are doing, unsubscribing from all
// filter lists.
SubscriptionUtil.validateUnderSubscription = function valUnderSubscription(clickedFitlerList) {
  const totalSelected = $('.filter_list_control:checked').length;
  const aaClicked = BG.isAcceptableAds(clickedFitlerList);
  const aaPrivacySelected = $('[name=acceptable_ads_privacy] :checked').length;

  // If the user unsubscribe to AA while the only other selection is AA Privacy
  // we want to show the dialog window because we're going to force the
  // unsubscription of AA Privacy and the totalSelection will actually be 0
  const aaPrivacyIsLast = totalSelected === 1 && aaClicked && aaPrivacySelected;

  if (totalSelected >= 1 && !(aaPrivacyIsLast)) {
    return true;
  }

  if (optionalSettings.show_advanced_options) {
    // In case of an advanced user, only warn once every 30 minutes, even
    // if the options page wasn't open all the time. 30 minutes = 1/48 day
    if (getStorageCookie('noUnderSubscriptionWarning')) {
      return true;
    }
    setStorageCookie('noUnderSubscriptionWarning', 'true', THIRTY_MINUTES_IN_MILLISECONDS);
  }

  // eslint-disable-next-line no-alert
  return window.confirm(translate('unsubscribe_from_all_confirmation'));
};

// Subscribe to the filter list with the given |id|.
// Input:
//   id:string - Id of the filter list to be subscribed to.
SubscriptionUtil.subscribe = function subscribeFunction(id, title) {
  const parameters = { id, title };
  const subscription = FilterListUtil.cachedSubscriptions[id];
  if (subscription && subscription.requiresList) {
    parameters.requires = subscription.requiresList;
  }

  SubscriptionUtil.updateCacheValue(id);
  BG.subscribe(parameters);
  if (BG.isAcceptableAds(subscription)) {
    updateAcceptableAdsUI(true, false);
  } else if (BG.isAcceptableAdsPrivacy(subscription)) {
    SubscriptionUtil.updateCacheValue('acceptable_ads');
    BG.unsubscribe({ id: 'acceptable_ads', del: false });
    updateAcceptableAdsUI(true, true);
  } else if ((id === 'easylist') || (id === 'easylist_lite')) {
    $('#easylist_info').slideUp();
  }
};

// Unsubscribe to the filter list with the given |id|.
// Input:
//   id:string - Id of the filter list to unsubscribe to.
//   del:boolean - Flag to indicate if filter list should be deleted in the background.
SubscriptionUtil.unsubscribe = function unsubscribeFunction(id, del) {
  const subscription = FilterListUtil.cachedSubscriptions[id];

  SubscriptionUtil.updateCacheValue(id);
  BG.unsubscribe({ id, del });
  if (BG.isAcceptableAds(subscription)) {
    SubscriptionUtil.updateCacheValue('acceptable_ads_privacy');
    BG.unsubscribe({ id: 'acceptable_ads_privacy', del: false });
    updateAcceptableAdsUI(false, false);
  } else if (BG.isAcceptableAdsPrivacy(subscription)) {
    SubscriptionUtil.updateCacheValue('acceptable_ads');
    BG.subscribe({ id: 'acceptable_ads' });
    updateAcceptableAdsUI(true, false);
  } else if (((id === 'easylist') || (id === 'easylist_lite'))
      && ($('li:has(:checkbox:checked)[name="easylist"], li:has(:checkbox:checked)[name="easylist_lite"]').length === 0)) {
    $('#easylist_info').slideDown();
  }
};

// Update the given filter list in the cached list.
// Input:
//   id:string - Id of the filter list to be updated.
SubscriptionUtil.updateCacheValue = function updateCacheValueFunction(id) {
  const sub = FilterListUtil.cachedSubscriptions[id];
  if (sub) {
    delete sub.lastUpdateFailedAt;
    delete sub.lastUpdate;
  }
};

// Perform the subscribing part and creating checkbox for custom filter lists.
// Inputs:
//   url:string - Url for the custom filter list.
//   subscribeTo:string - The id of the custom filter list.
CustomFilterListUploadUtil.performUpload = function uploadFunction(url, subscribeTo, title) {
  SubscriptionUtil.subscribe(subscribeTo, title);
  const entry = {
    id: subscribeTo,
    url,
    subscribed: true,
    unsubscribe: true,
    userSubmitted: true,
    label: title || '',
  };
  FilterListUtil.cachedSubscriptions[entry.id] = entry;
  const { customFilterList } = filterListSections;
  const checkbox = new CheckboxForFilterList(
    entry,
    'customFilterList', customFilterList.array.length, customFilterList.container,
  );
  checkbox.createCheckbox(true);
};

// When a user enters a URL in the custom filter list textbox for a known filter list,
// this method clicks the correct filter list checkbox/select option for him instead.
// Input:
//   existingFilterList:object - Filter list whose URL was entered by the user.
// eslint-disable-next-line max-len
CustomFilterListUploadUtil.updateExistingFilterList = function updateExistingFunction(existingFilterList) {
  let containingDiv = $(`li[name='${existingFilterList.id}']`);
  if (containingDiv.length < 1) {
    // If the checkbox does not exist but there is an existing filter list,
    // then recreate the checkbox
    const filterListType = FilterListUtil.getFilterListType(existingFilterList);
    const filterListArray = filterListSections[filterListType].array;
    let index = filterListArray.indexOf(existingFilterList);
    if (index < 0) {
      index = filterListArray.length;
      filterListArray.push(existingFilterList);
    }

    const checkboxForFilterList = new CheckboxForFilterList(
      existingFilterList,
      filterListType, index, filterListSections[filterListType].container,
    );
    checkboxForFilterList.createCheckbox();
    containingDiv = $(`div[name='${existingFilterList.id}']`);
  }

  const checkbox = $(containingDiv).find('input');

  if (!checkbox.is(':checked')) {
    const checkboxID = checkbox.attr('id');
    if (checkboxID && checkboxID.indexOf('languageFilterList') >= 0) {
      LanguageSelectUtil.triggerChange(existingFilterList);
    } else {
      checkbox.prop('checked', 'checked');
      checkbox.trigger('change');
    }
  }
};

// Binds events for key press 'enter' and click for upload box.
CustomFilterListUploadUtil.bindControls = function bindControlsFunction() {
  $('#btnNewSubscriptionUrl').click(() => {
    let url = $('#txtNewSubscriptionUrl').val();
    const abpRegex = /^abp.*\Wlocation=([^&]+)/i;
    if (abpRegex.test(url)) {
      [, url] = url.match(abpRegex); // The part after 'location='.
      url = unescape(url);
    }

    url = url.trim();
    const subscribeTo = `url:${url}`;
    const existingFilterList = FilterListUtil.checkUrlForExistingFilterList(url);

    if (existingFilterList) {
      CustomFilterListUploadUtil.updateExistingFilterList(existingFilterList);
    } else if (/^https?:\/\/[^<]+$/.test(url)) {
      CustomFilterListUploadUtil.performUpload(url, subscribeTo);
    } else {
      // eslint-disable-next-line no-alert
      alert(translate('failedtofetchfilter'));
    }

    $('#txtNewSubscriptionUrl').val('');
  });

  // Pressing enter will add the list too.
  $('#txtNewSubscriptionUrl').keypress((event) => {
    if (event.keyCode === 13) {
      event.preventDefault();
      $('#btnNewSubscriptionUrl').click();
    }
  });
};

const filtersInit = function () {
  // Retrieves list of filter lists from the background.
  let subs = BG.getSubscriptionsMinusText();
  // Initialize page using subscriptions from the background.
  // Copy from update subscription list + setsubscriptionlist

  filterListSections = {
    adblockFilterList: {
      array: [],
      container: $('#ad_blocking_list'),
    },
    languageFilterList: {
      array: [],
      container: $('#language_list'),
    },
    otherFilterList: {
      array: [],
      container: $('#other_filter_lists'),
    },
    customFilterList: {
      array: [],
      container: $('#custom_filter_lists'),
    },
  };

  FilterListUtil.prepareSubscriptions(subs);

  for (const id in filterListSections) {
    const sectionHandler = new SectionHandler(filterListSections[id], id);
    sectionHandler.initSection();
  }

  LanguageSelectUtil.init();

  CustomFilterListUploadUtil.bindControls();

  // if the user is subscribed to malware, add the checkbox for notifications
  if (subs && subs.malware && subs.malware.subscribed) {
    addMalwareNotificationDiv();
  }

  window.setInterval(() => {
    FilterListUtil.updateSubscriptionInfoAll();
  }, 5000);

  $('#btnUpdateNow').click(() => {
    $(this).attr('disabled', 'disabled');
    BG.updateSubscriptionsNow();
    setTimeout(() => {
      $('#btnUpdateNow').removeAttr('disabled');
    }, 300000); // Re-enable update button after 5 minutes.
  });

  $('#btnShowLinks').click(() => {
    $('.linkToList').fadeIn('slow');
    $('#btnShowLinks').remove();
  });

  if (FilterListUtil.cachedSubscriptions.easylist.subscribed
    && FilterListUtil.cachedSubscriptions.easylist_lite.subscribed) {
    $('#easylist_lite_both_info').slideDown();
  } else {
    $('#easylist_lite_both_info').slideUp();
  }

  browser.runtime.onMessage.addListener((request, sender, sendResponse) => {
    if (request && request.command !== 'filters_updated') {
      return;
    }

    if (!request) {
      return;
    }

    subs = BG.getSubscriptionsMinusText();
    const { cachedSubscriptions } = FilterListUtil;
    for (const id in cachedSubscriptions) {
      const entry = subs[id];
      if (entry) {
        // Update checkbox according to the value of the subscribed field
        FilterListUtil.updateCheckbox(entry, id);
        if (cachedSubscriptions[id]) {
          cachedSubscriptions[id].subscribed = entry.subscribed;
        }
        // If entry is subscribed, update lastUpdateFailedAt and lastUpdate field
        if (entry.subscribed) {
          if (entry.lastUpdate && entry.lastUpdateFailedAt) {
            // If update is more recent than failed update, remove lastUpdateFailedAt field,
            // otherwise, remove lastUpdate field
            if (parseInt(entry.lastUpdate, 10) > parseInt(entry.lastUpdateFailedAt, 10)) {
              delete subs[id].lastUpdateFailedAt;
            } else {
              delete subs[id].lastUpdate;
            }
          }

          // Update lastUpdateFailedAt and lastUpdate field for the entry in cached subscriptions
          if (entry.lastUpdateFailedAt) {
            cachedSubscriptions[id].lastUpdateFailedAt = entry.lastUpdateFailedAt;
          } else if (entry.lastUpdate) {
            cachedSubscriptions[id].lastUpdate = entry.lastUpdate;
          }
        } else {
          SubscriptionUtil.updateCacheValue(id);
        }
      }
    }

    if (FilterListUtil.cachedSubscriptions.easylist.subscribed
      && FilterListUtil.cachedSubscriptions.easylist_lite.subscribed) {
      $('#easylist_lite_both_info').slideDown();
    } else {
      $('#easylist_lite_both_info').slideUp();
    }

    sendResponse({});
  });
};

$(document).ready(() => {
  const $txtInputCustomURL = $('#txtNewSubscriptionUrl');
  $txtInputCustomURL.attr('placeholder', translate('enter_url'));
});
